const { Adw, Gio, GLib, GObject } = imports.gi;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const FileUtils = imports.misc.fileUtils;
const gettextDomain = Me.metadata["gettext-domain"];
const Gettext = imports.gettext.domain(gettextDomain);
const _ = Gettext.gettext;

const POWER_PROFILES_BUS_NAME = "net.hadess.PowerProfiles";
const POWER_PROFILES_OBJECT_PATH = "/net/hadess/PowerProfiles";

const PowerProfilesIface = FileUtils.loadInterfaceXML(
  "net.hadess.PowerProfiles"
);
const PowerProfilesProxy = Gio.DBusProxy.makeProxyWrapper(PowerProfilesIface);

async function execCheck(argv) {
  const flags =
    Gio.SubprocessFlags.STDOUT_PIPE | Gio.SubprocessFlags.STDERR_PIPE;

  const proc = new Gio.Subprocess({
    argv,
    flags,
  });
  proc.init(null);

  return new Promise((resolve, reject) => {
    proc.communicate_utf8_async(null, null, (obj, res) => {
      try {
        const [, stdout, stderr] = obj.communicate_utf8_finish(res);
        const status = obj.get_exit_status();
        resolve([status, stdout]);
      } catch (e) {
        reject(e);
      }
    });
  });
}

function bindAdwComboRow(comboRow, settings, key, map_) {
  const initValue = settings.get_string(key);
  comboRow.selected = map_.indexOf(initValue);

  settings.connect(`changed::${key}`, () => {
    const idx = map_.indexOf(settings.get_string(key));
    comboRow.selected = idx;
  });
  comboRow.connect("notify::selected", () => {
    const value = map_[comboRow.selected];
    settings.set_string(key, value);
  });
}

const getIcon = (driver) =>
  driver && driver !== "placeholder"
    ? { name: "emblem-default-symbolic", styleclass: "success" }
    : { name: "emblem-important-symbolic", styleclass: "error" };

var General = GObject.registerClass(
  {
    GTypeName: "AutoPowerProfileGeneralPrefs",
    Template: `file://${GLib.build_filenamev([Me.path, "ui", "general.ui"])}`,
    InternalChildren: [
      "ac_profile",
      "bat_profile",
      "threshold",

      "row_version",
      "row_cpu_driver",
      "row_platform_driver",
      "version_icon",
      "platform_driver_icon",
      "cpu_driver_icon",

      "ppd_version_label",
      "platform_driver_label",
      "cpu_driver_label",
      "debug_logging",
      "platform_profile_model",
    ],
  },
  class General extends Adw.PreferencesPage {
    _init(settings, params = {}) {
      super._init(params);

      const PROFILES_I18N = [
        ["performance", _("Performance")],
        ["balanced", _("Balanced")],
        ["power-saver", _("Power Saver")],
      ];

      settings.bind(
        "threshold",
        this._threshold,
        "value",
        Gio.SettingsBindFlags.DEFAULT
      );

      settings.bind(
        "debug",
        this._debug_logging,
        "active",
        Gio.SettingsBindFlags.DEFAULT
      );

      this._powerProfilesProxy = new PowerProfilesProxy(
        Gio.DBus.system,
        POWER_PROFILES_BUS_NAME,
        POWER_PROFILES_OBJECT_PATH,
        async (proxy, error) => {
          if (error) {
            console.error(error.message);

            this._version_icon.icon_name = "emblem-important-symbolic";
            this._version_icon.add_css_class("error");

            this._platform_driver_icon.icon_name = "emblem-important-symbolic";
            this._platform_driver_icon.add_css_class("error");

            this._cpu_driver_icon.icon_name = "emblem-important-symbolic";
            this._cpu_driver_icon.add_css_class("error");
            return;
          }

          const profileKeys = this._powerProfilesProxy.Profiles?.map((x) =>
            x.Profile.unpack()
          );
          const availableProfiles = PROFILES_I18N.filter(([k, name]) =>
            profileKeys.includes(k)
          );
          const indexedProfiles = availableProfiles.map(([k, name]) => k);

          availableProfiles.forEach(([k, name]) => {
            this._platform_profile_model.append(name);
          });
          bindAdwComboRow(this._ac_profile, settings, "ac", indexedProfiles);
          bindAdwComboRow(this._bat_profile, settings, "bat", indexedProfiles);

          let ver = "";

          // try to detect PPD version
          try {
            const [version] = this._powerProfilesProxy
              .get_cached_property("Version")
              .get_string();
            ver = version;
          } catch (e) {
            console.error(e);
            try {
              const [status, stdout] = await execCheck([
                "/usr/bin/powerprofilesctl",
                "version",
              ]);
              ver = stdout.replace(/\s/, "");
            } catch (e1) {
              console.error(e1);
            }
          } finally {
            if (ver) {
              this._ppd_version_label.label = ver;
              this._version_icon.icon_name = "emblem-default-symbolic";
              this._version_icon.add_css_class("success");
            } else {
              this._version_icon.icon_name = "emblem-important-symbolic";
              this._version_icon.add_css_class("error");
            }
          }

          // try to get PPD drivers info
          const active = this._powerProfilesProxy.ActiveProfile;
          const profile = this._powerProfilesProxy.Profiles.find(
            (x) => x.Profile.unpack() === active
          );
          try {
            const [platformDriver] = profile.PlatformDriver.get_string();
            const platform_icon = getIcon(platformDriver);
            this._platform_driver_icon.icon_name = platform_icon.name;
            this._platform_driver_icon.add_css_class(platform_icon.styleclass);
            this._platform_driver_label.label = platformDriver;
          } catch (e) {
            console.error(e);

            try {
              const [Driver] = profile.Driver.get_string();
              const platform_icon = getIcon(Driver);
              this._platform_driver_icon.icon_name = platform_icon.name;
              this._platform_driver_icon.add_css_class(
                platform_icon.styleclass
              );
              this._platform_driver_label.label = Driver;
            } catch (e) {
              console.error(e);
            }
          }

          try {
            const [cpuDriver] = profile.CpuDriver.get_string();
            const cpu_icon = getIcon(cpuDriver);
            this._cpu_driver_icon.icon_name = cpu_icon.name;
            this._cpu_driver_icon.add_css_class(cpu_icon.styleclass);
            this._cpu_driver_label.label = cpuDriver;
          } catch (e) {
            console.error(e);
            this._row_cpu_driver.visible = false;
          }
        }
      );
    }
  }
);

function init() {
  ExtensionUtils.initTranslations(Me.metadata.uuid);
}

function fillPreferencesWindow(window) {
  const settings = ExtensionUtils.getSettings(
    "org.gnome.shell.extensions.auto-power-profile"
  );
  window.add(new General(settings));
}
